/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: ValidatorPool.java 5226 2009-11-08 18:40:34Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.enhanced.impl.validator;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.NotABusyPoolItemException;
import org.ow2.util.pool.impl.enhanced.api.PoolException;
import org.ow2.util.pool.impl.enhanced.api.TimeoutPoolException;
import org.ow2.util.pool.impl.enhanced.api.validator.IPoolValidator;
import org.ow2.util.pool.impl.enhanced.impl.PoolError;

/**
 * @author Gael Lalire
 * @param <E> pool item type
 */
public class ValidatorPool<E> implements IPool<E> {

    private static final Log LOG = LogFactory.getLog(ValidatorPool.class);

    private IPool<E> pool;

    private IPoolValidator<E> poolValidator;

    private boolean putValidate;

    public ValidatorPool(final IPool<E> pool, final IPoolValidator<E> poolValidator, final boolean putValidate) {
        this.pool = pool;
        this.poolValidator = poolValidator;
        this.putValidate = putValidate;
    }

    protected boolean validateOneItem(final E poolItem) {
        try {
            return poolValidator.validate(poolItem);
        } catch (RuntimeException e) {
            LOG.error("Validation failed", e);
            return false;
        }
    }

    public E get(final IWaitControl timeout) throws PoolException, InterruptedException {
        E poolItem = pool.get(timeout);
        while (!validateOneItem(poolItem)) {
            try {
                pool.remove(poolItem);
            } catch (NotABusyPoolItemException e) {
                throw new PoolError("Delegated pool has a bad behavior");
            }
            if (!timeout.canContinueToWait()) {
                throw new TimeoutPoolException();
            }
            poolItem = pool.get(timeout);
        }
        return poolItem;
    }

    public void signalAllWaiters() {
        pool.signalAllWaiters();
    }

    public void put(final E poolItem) throws NotABusyPoolItemException {
        if (putValidate && !validateOneItem(poolItem)) {
            pool.remove(poolItem);
        } else {
            pool.put(poolItem);
        }
    }

    public void remove(final E poolItem) throws NotABusyPoolItemException {
        pool.remove(poolItem);
    }

}
